#!/usr/bin/env node
'use strict';

var _path = _interopRequireDefault(require("path"));

var _chalk = _interopRequireDefault(require("chalk"));

var _gaze = _interopRequireDefault(require("gaze"));

var _globby = _interopRequireDefault(require("globby"));

var _meow = _interopRequireDefault(require("meow"));

var _converter = _interopRequireDefault(require("./converter"));

var _writer = _interopRequireDefault(require("./writer"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var cli = (0, _meow.default)({
  description: 'Creates .flow type definition files from CSS Modules files',
  help: `
Usage
  $ css-modules-flow-types <path> [<path>] [options]

path    directory to search for CSS Modules (or concrete files to convert)

Options
  --watch, -w       Run in watch mode
  --extension, -e   File extension (defaults to "css")
  --silent, -s      Silences all output except errors
    `,
  inferType: true
}, {
  flags: {
    _: {
      type: 'string'
    },
    extension: {
      alias: 'e'
    },
    help: {
      alias: 'h'
    },
    silent: {
      alias: 's',
      type: 'boolean'
    },
    watch: {
      alias: 'w',
      type: 'boolean'
    }
  }
});

function detectDanlingFlowFiles(filesPattern, cssFiles) {
  var flowFiles = _globby.default.sync(filesPattern + '.flow');

  var cssFilesSet = new Set(cssFiles);
  var danglingFlowFiles = flowFiles.filter(function (f) {
    return !cssFilesSet.has(f.replace('.flow', ''));
  });

  if (danglingFlowFiles.length > 0) {
    console.error(_chalk.default.red(`Detected ${danglingFlowFiles.length} dangling .flow file(s), that can be removed:`));
    danglingFlowFiles.forEach(function (f) {
      console.error(_chalk.default.red(`- ${f}`));
    });
  }
}

var main = function main() {
  var _cli$flags = cli.flags,
      watch = _cli$flags.watch,
      silent = _cli$flags.silent;

  if (!cli.input || cli.input.length === 0) {
    return cli.showHelp();
  }

  var extension = cli.flags.extension || 'css';
  var filesList = cli.input.length > 1 ? cli.input : null;
  var filePath = cli.input.length === 1 ? cli.input[0] : null;

  var filesPattern = filePath && _path.default.join(filePath, `**/*.${extension}`);

  var rootDir = process.cwd();
  var converter = new _converter.default(rootDir);

  function handleFile(filePath) {
    var f = _path.default.resolve(filePath);

    converter.convert(f).then(function (content) {
      var outputFilePath = _path.default.join(f + '.flow');

      return (0, _writer.default)(outputFilePath, content);
    }).then(function (outputFilePath) {
      if (!silent) {
        console.log('Wrote ' + _chalk.default.green(outputFilePath));
      }
    }).catch(function (reason) {
      return console.error(_chalk.default.red('[Error] ' + reason));
    });
  }

  if (!watch) {
    var cssFiles = filesList ? filesList : _globby.default.sync(filesPattern);
    cssFiles.forEach(handleFile);

    if (!filesList) {
      detectDanlingFlowFiles(filesPattern, cssFiles);
    }
  } else {
    if (!filePath) {
      console.error(_chalk.default.red(`Watch mode requires a single path... Not ${filesList}`));
      return;
    }

    (0, _gaze.default)(filesPattern, function (err, files) {
      this.on('changed', handleFile);
      this.on('added', handleFile);
    });
  }
};

main();